@@ -161,6 +161,32 @@ to_sql.relop_orderby <- function (x,
                                   tnum = mk_tmp_name_source('tsql'),
                                   append_cr = TRUE,
                                   using = NULL) {
+  if(length(list(...))>0) {
+    stop("rquery::to_sql.relop_orderby unexpected arguments")
+  }
+  dispatch_to_sql_method(
+    method_name = "to_sql.relop_orderby",
+    x = x,
+    db = db,
+    limit = limit,
+    source_limit = source_limit,
+    indent_level = indent_level,
+    tnum = tnum,
+    append_cr = append_cr,
+    using = using)
+}
+
+
+to_sql_relop_orderby <- function(
+  x,
+  db,
+  ...,
+  limit = NULL,
+  source_limit = NULL,
+  indent_level = 0,
+  tnum = mk_tmp_name_source('tsql'),
+  append_cr = TRUE,
+  using = NULL) {
   wrapr::stop_if_dot_args(substitute(list(...)),
                           "rquery::to_sql.relop_orderby")
   cols1 <- column_names(x$source[[1]])
@@ -188,12 +214,12 @@ to_sql.relop_orderby <- function (x,
   tab <- tnum()
   prefix <- paste(rep(' ', indent_level), collapse = '')
   q <- paste0(prefix, "SELECT * FROM (\n",
-         subsql, "\n",
-         prefix, ") ",
-         tab,
-         ifelse(length(ot)>0,
-                paste0(" ORDER BY ", paste(ot, collapse = ", ")),
-                ""))
+              subsql, "\n",
+              prefix, ") ",
+              tab,
+              ifelse(length(ot)>0,
+                     paste0(" ORDER BY ", paste(ot, collapse = ", ")),
+                     ""))
   if(!is.null(x$limit)) {
     limit <- min(limit, x$limit)
   }
