@@ -191,6 +191,31 @@ to_sql.relop_select_rows <- function (x,
                                       tnum = mk_tmp_name_source('tsql'),
                                       append_cr = TRUE,
                                       using = NULL) {
+  if(length(list(...))>0) {
+    stop("rquery::to_sql.relop_select_rows unexpected arguments")
+  }
+  dispatch_to_sql_method(
+    method_name = "to_sql.relop_select_rows",
+    x = x,
+    db = db,
+    limit = limit,
+    source_limit = source_limit,
+    indent_level = indent_level,
+    tnum = tnum,
+    append_cr = append_cr,
+    using = using)
+}
+
+to_sql_relop_select_rows <- function(
+  x,
+  db,
+  ...,
+  limit = NULL,
+  source_limit = NULL,
+  indent_level = 0,
+  tnum = mk_tmp_name_source('tsql'),
+  append_cr = TRUE,
+  using = NULL) {
   wrapr::stop_if_dot_args(substitute(list(...)),
                           "rquery::to_sql.relop_select_rows")
   # re-quote expr
@@ -211,11 +236,11 @@ to_sql.relop_select_rows <- function (x,
   tab <- tnum()
   prefix <- paste(rep(' ', indent_level), collapse = '')
   q <- paste0(prefix, "SELECT * FROM (\n",
-         subsql, "\n",
-         prefix, ") ",
-         tab, "\n",
-         prefix, "WHERE ",
-         re_expr)
+              subsql, "\n",
+              prefix, ") ",
+              tab, "\n",
+              prefix, "WHERE ",
+              re_expr)
   if(!is.null(limit)) {
     q <- paste(q, "LIMIT",
                format(ceiling(limit), scientific = FALSE))
@@ -225,3 +250,4 @@ to_sql.relop_select_rows <- function (x,
   }
   c(subsql_list[-length(subsql_list)], q)
 }
+
